unit GPScgw;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  {System.Variants, FMX.Types, FMX.Controls,}
  FMX.Forms,
  {FMX.Graphics, FMX.Dialogs,}
  Androidapi.JNI.Location, Androidapi.JNIBridge, Androidapi.JNI.JavaTypes,
  Androidapi.JNI.Os, {FMX.Layouts, FMX.ListBox, FMX.StdCtrls,}
  FMX.Helpers.Android, AndroidAPI.Looper,
  Androidapi.Helpers, Androidapi.JNI.GraphicsContentViewText,
  System.Math;

type
  TGPScgwForm = class(TForm)
  private
    { Private declarations }
  public
    { Public declarations }
  end;


  type
  TLocationListener = class(TJavaLocal, JLocationListener)
  private
	 [weak]
	 FParent : TForm;
  public
	 constructor Create(AParent : TForm);
	 procedure onLocationChanged(location: JLocation); overload; cdecl;
	 procedure onProviderDisabled(provider: JString); cdecl;
	 procedure onProviderEnabled(provider: JString); cdecl;
	 procedure onStatusChanged(provider: JString; status: Integer; extras: JBundle); cdecl;
  private
		fLatitude, fLongitude, fAltitude : double;
		fSpeed : single;
  end;



  tGPScgw = class(tComponent)
	  constructor Create(AParent : tComponent); override;
	  destructor Destroy; override;
	  procedure Start(Form : tForm; RefreshTime : integer);
	  procedure Stop;

	  private
		  LocationManagerService: JObject;
		  FLocationManager : JLocationManager;
		  LocationListener : TLocationListener;
		  function GetLat : double;
		  function GetLong : double;
		  function GetAlt : double;
		  function GetSpeed : single;

	  public
		  property Latitude : double read GetLat;
		  property Longitude : double read GetLong;
		  property Altitude : double read GetAlt;
		  property Speed : single read GetSpeed;
  end;

var
   GPScgwForm : tGPScgwForm;

	 tempint1, tempint2, tempint3, tempint4 : integer;
	 tempbool : boolean;

implementation

{$R *.fmx}


{ tGPScgw }

constructor tGPScgw.Create(AParent : tComponent);
begin
	 inherited Create(AParent);
	 FLocationManager := nil;
	 LocationListener := tLocationListener.Create(GPScgwForm);
end;

destructor tGPScgw.Destroy;
begin
	 Stop;
	 inherited Destroy;
end;

procedure tGPScgw.Start(Form : tForm; RefreshTime : integer);
begin
	 LocationManagerService := tAndroidHelper.Context.getSystemService(
      TJContext.JavaClass.LOCATION_SERVICE);
	 FLocationManager := TJLocationManager.Wrap(
      (LocationManagerService as ILocalObject).GetObjectID);

	 FLocationManager.requestLocationUpdates(
			TJLocationManager.JavaClass.GPS_PROVIDER, RefreshTime, 0, LocationListener,
			TJLooper.JavaClass.getMainLooper);
end;

procedure tGPScgw.Stop;
begin
	 if Assigned(fLocationManager) then
	    FLocationManager.removeUpdates(LocationListener);
end;

function tGPScgw.GetLat : double;
begin
	 Result := LocationListener.fLatitude;
end;

function tGPScgw.GetLong : double;
begin
	 Result := LocationListener.fLongitude;
end;

function tGPScgw.GetAlt : double;
begin
	 Result := LocationListener.fAltitude;
end;

function tGPScgw.GetSpeed : single;
begin
// Result := SpeedoMainForm.LocationSensor1.Sensor.Speed;     FORGET IT!  LocationSensors are hopeless.

   Result := LocationListener.fSpeed;
   if IsNan(Result) then Result := -1;
end;


{ TLocationListener }

constructor TLocationListener.Create(AParent: TForm);
begin
   inherited Create;
   FParent := AParent;
end;

procedure TLocationListener.onLocationChanged(location: JLocation);
begin
   fLatitude := Location.getLatitude;
   fLongitude := Location.getLongitude;
	 fAltitude := Location.getAltitude;
	 fSpeed := Location.getSpeed;
end;

procedure TLocationListener.onProviderDisabled(provider: JString);
begin

end;

procedure TLocationListener.onProviderEnabled(provider: JString);
begin

end;

procedure TLocationListener.onStatusChanged(provider: JString; status: Integer;
  extras: JBundle);
begin

end;

end.

