unit SpeedoMainUnit;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Platform.Android, FMX.Objects, FMX.StdCtrls, System.IOUtils,
  System.Sensors, System.Sensors.Components, System.Math, PowerManager,
  FMX.Controls.Presentation, FMX.Edit, FMX.Layouts, FMX.Memo, FMX.ScrollBox,
  FMX.Memo.Types, System.Permissions, FMX.DialogService, gpsCGW;

type
  TSpeedoMainForm = class(TForm)
	 ImageSpeed: TImage;
    Timer1: TTimer;
    LocationSensor1: TLocationSensor;
    Memo1: TMemo;
	 function SpeedFmt : string;

	 procedure FormCreate(Sender: TObject);
	 procedure FormResize(Sender: TObject);
	 procedure FormActivate(Sender: TObject);
	 procedure FormClose(Sender: TObject; var Action: TCloseAction);
	 procedure Timer1Timer(Sender: TObject);
  private
	 { Private declarations }
  public
	 { Public declarations }
  end;

var
	SpeedoMainForm: TSpeedoMainForm;
  count : integer;

	gSpeed: double;

	HeightOfSpeedText, WidthOfSpeedText : integer;

	Initialized : boolean;

	tempint1, tempint2, tempint3, tempint4 : integer;
	tempstr1, tempstr2, tempstr3, tempstr4 : string;
  tempBool : boolean;

  GPS : tGPScgw;

	const DataRate = 500;

implementation

{$R *.fmx}


procedure TSpeedoMainForm.FormCreate(Sender: TObject);
begin
   count := 0;

   Canvas.Font.Family := 'Tahoma';
   Canvas.Font.Size := 24; //96;
   Canvas.Font.Style := [tFontStyle.fsBold];

   Initialized := false;
end;


procedure TSpeedoMainForm.FormActivate(Sender: TObject);
var
   PermissionAccessFineLocation : string;
begin
	 if Initialized then exit;
   Initialized := true;

   AcquireWakeLock;  //Keep the screen awake.

   ImageSpeed.Bitmap := tBitmap.Create(round(ImageSpeed.Width), round(ImageSpeed.Height));

   Memo1.Lines.Clear;

  PermissionAccessFineLocation := 'android.permission.ACCESS_FINE_LOCATION';

   PermissionsService.RequestPermissions([PermissionAccessFineLocation],
      procedure(const APermissions: TClassicStringDynArray;
                const AGrantResults: TClassicPermissionStatusDynArray)
      begin
         if (Length(AGrantResults) = 1) and (AGrantResults[0] = TPermissionStatus.Granted) then
            { activate or deactivate the location sensor }
            LocationSensor1.Active := true
      else
         begin
            TDialogService.ShowMessage('Location permission not granted');
         end;
      end);

   LocationSensor1.Active := true;
   LocationSensor1.Sensor.Start;

	 GPS := tGPScgw.Create(SpeedoMainForm);
	 GPS.Start(SpeedoMainForm, DataRate);

   Timer1.Enabled := true;
end;

procedure TSpeedoMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	ReleaseWakeLock;
end;


procedure TSpeedoMainForm.Timer1Timer(Sender: TObject);
var
  s : string;
begin
   while Memo1.Lines.Count > 5 do Memo1.Lines.Delete(0);

   gSpeed := GPS.Speed;
   inc(count);
   Memo1.Lines.Add('Speed: T (' + IntToStr(count) + '):' + FloatToStr(gSpeed));
   Memo1.Repaint;

   if gSpeed < 0 then exit;

	 gSpeed := gSpeed * 3.6;   // to km/h

   ImageSpeed.Bitmap.Canvas.Font.Family := 'Tahoma';
   ImageSpeed.Bitmap.Canvas.Font.Style := [tFontStyle.fsBold];

   ImageSpeed.Position.X := 0;
   ImageSpeed.Position.Y := 0;

   s := SpeedFmt;
   ImageSpeed.Bitmap.Canvas.Font.Size := 300;
   while ImageSpeed.Bitmap.Canvas.TextWidth(s) > ImageSpeed.Width do
     	ImageSpeed.Bitmap.Canvas.Font.Size := ImageSpeed.Bitmap.Canvas.Font.Size - 20;


   ImageSpeed.Bitmap.Canvas.BeginScene;

   ImageSpeed.Bitmap.Canvas.Fill.Color :=  (tAlphaColorRec.Yellow or $c0);
   ImageSpeed.Bitmap.Canvas.Clear(TAlphaColorRec.Black);

   ImageSpeed.Bitmap.Canvas.FillText(
      tRectF.Create(0, 0, ImageSpeed.Width, ImageSpeed.Height),
	    s, false, 100, [TFillTextFlag.RightToLeft], TTextAlign.Center, TTextAlign.Center);

   ImageSpeed.Bitmap.Canvas.EndScene;
end;



function tSpeedoMainForm.SpeedFmt : string;
begin
   Result := '-';
   if isNan(gSpeed) then exit;

   if gSpeed < 0.05 then Result := '0'
   else
   begin
      if trunc(gSpeed) < 10 then
			   Result := Format('%5.1f', [gSpeed])
		  else
			   Result := Format('%3.0f', [gSpeed]);

      Result := trim(Result);
	 end;
end;




procedure TSpeedoMainForm.FormResize(Sender: TObject);
var
   s : string;
begin
	 ImageSpeed.Width := (ClientWidth);
	 ImageSpeed.Height := ClientHeight - Memo1.Height;		// (but not always!)
	 ImageSpeed.Height := (ClientHeight);

	 ImageSpeed.Bitmap.SetSize(round(ClientWidth), round(ClientHeight - Memo1.Height));
	 ImageSpeed.Bitmap.SetSize(round(ClientWidth), round(ClientHeight));

	 Memo1.Position.Y := ClientHeight - Memo1.Height;

   if (ImageSpeed.Bitmap.HandleAllocated and Initialized) then    //At the start there ain't no handle, causing things to crash.
   begin
    	gSpeed := GPS.Speed;
      inc(count);
      Memo1.Lines.Add('Speed: R (' + IntToStr(count) + '):' + FloatToStr(gSpeed));
    	gSpeed := gSpeed * 3.6;   // to km/h

    	s := SpeedFmt;

    	ImageSpeed.Bitmap.Canvas.Font.Family := 'Tahoma';
    	ImageSpeed.Bitmap.Canvas.Font.Style := [tFontStyle.fsBold];

    	ImageSpeed.Bitmap.Canvas.Font.Size := 300;
    	while ImageSpeed.Bitmap.Canvas.TextWidth(s) > ImageSpeed.Width do
         	ImageSpeed.Bitmap.Canvas.Font.Size := ImageSpeed.Bitmap.Canvas.Font.Size - 20;


    	ImageSpeed.Bitmap.Canvas.BeginScene;

    	ImageSpeed.Bitmap.Canvas.Fill.Color :=  (tAlphaColorRec.Yellow or $c0);
    	ImageSpeed.Bitmap.Canvas.Clear(TAlphaColorRec.Black);

    	ImageSpeed.Bitmap.Canvas.FillText(tRectF.Create(0, 0, ImageSpeed.Width, ImageSpeed.Height),
      		s, false, 100, [TFillTextFlag.RightToLeft], TTextAlign.Center, TTextAlign.Center);

    	ImageSpeed.Bitmap.Canvas.EndScene;
	end;
end;

end.


